#!/usr/bin/env ts-node

import { Command } from 'commander';
import simpleGit, { SimpleGit } from 'simple-git';
import * as path from 'path';
import * as fs from 'fs';
import inquirer from 'inquirer';
import fsx from 'fs/promises';
import { spawn } from 'child_process';

const program = new Command();
program.version('1.0.0');

interface Options {
    dir?: string;
    name: string;
    branch?: string;
}

program
    .option('-d, --dir <dir>', '项目目录，默认当前文件夹', process.cwd())
    .option('-n, --name <name>', '模板名称，对应 Gitee group 下的项目名称', '')
    .option('-b, --branch <branch>', '分支名称，默认 main 分支', 'main');

program.parse(process.argv);

const git: SimpleGit = simpleGit();
// 输入项目目录
async function selectProjectDir() {
    let projectDir;

    // 使用一个循环来持续请求用户输入，直到得到有效的目录
    while (!projectDir) {
        const answers = await inquirer.prompt([
            {
                type: 'input',
                name: 'projectDir',
                message: '请输入项目目录路径：',
                // 提供一个默认值，例如当前工作目录
                default: process.cwd(),
                // 验证用户输入
                validate: (input) => {
                    const resolvedPath = path.resolve(input);
                    if (!fs.existsSync(resolvedPath) || !fs.statSync(resolvedPath).isDirectory()) {
                        return '目录不存在或不是一个有效的目录，请输入一个有效的目录路径。';
                    }
                    // 如果目录存在且是有效的，返回 true 以结束验证
                    return true;
                },
            },
        ]);

        // 如果验证通过，则赋值给 projectDir
        if (answers.projectDir) {
            projectDir = answers.projectDir;
        }
    }

    // 返回有效的项目目录路径
    return projectDir;
}
//选择模版类型==》vue、react
async function selectTemplate() {
    // 读取 config/templates.json 文件
    const templatesFilePath = path.join(__dirname, '..', 'config', 'templates.json');
    try {
        const templatesData = await fsx.readFile(templatesFilePath, 'utf8');
        const { templates } = JSON.parse(templatesData)
        // 使用 inquirer 提示用户选择模板
        const { template } = await inquirer.prompt([
            {
                type: 'list',
                name: 'template',
                message: '请选择一个模板:',
                choices: templates,
            },
        ]);

        return template;
    } catch (err) {
        console.error('读取模板配置文件时出错:', err);
        throw err;
    }
}
// async function selectTemplate() {
//     const templates = [
//       { name: 'Vue', value: 'http://code.devops.piccnet/picc/picc__picc-wechat-sale-platform-etong/enterprise-insurance-management.git' },
//       { name: 'React', value: 'http://code.devops.piccnet/picc/picc__picc-wechat-sale-platform-etong/downloadapks_pad.git' },
//       // ...更多模板
//     ];
   
//     const { template } = await inquirer.prompt([
//       {
//         type: 'list',
//         name: 'template',
//         message: '请选择一个模板:',
//         choices: templates,
//       },
//     ]);
//     return template;
//   } 
//选择分支
async function selectBranch() {
    // 读取 config/branches.json 文件
    const branchesFilePath = path.join(__dirname, '..', 'config', 'branchs.json');
    try {
        const branchesData = await fsx.readFile(branchesFilePath, 'utf8');
        const { branches } = JSON.parse(branchesData);

        // 使用 inquirer 提示用户选择分支
        const { branch } = await inquirer.prompt([
            {
                type: 'list',
                name: 'branch',
                message: '请选择一个分支:',
                choices: branches,
            },
        ]);
        return branch;
    } catch (err) {
        console.error('读取分支配置文件时出错:', err);
        throw err;
    }
}
// async function selectBranch() {
//     const branchList = [
//       { name: 'main', value: 'main' },
//       { name: 'starter-template', value: 'starter-template' },
//       // ...更多分支
//     ];
//     const { branch } = await inquirer.prompt([
//       {
//         type: 'list',
//         name: 'branch',
//         message: '请选择一个分支:',
//         choices: branchList,
//       },
//     ]);
//     return branch;
// }
// 克隆模版到指定项目目录
async function cloneRepo() {
    // 让用户输入项目目录路径
    const projectDir = await selectProjectDir();
    // 获取用户选择的模板
    const repoUrl = await selectTemplate();
    // 获取用户选择的分支
    const branch = await selectBranch();
    console.log(`正在克隆仓库 ${repoUrl} 到 ${projectDir} 分支 ${branch}`)
    // git.clone(repoUrl, projectDir, { '--branch': branch })
    //     .then(() => {
    //         console.log(`项目已成功克隆到 ${projectDir}`);
    //     })
    //     .catch((err) => {
    //         console.error(`克隆项目时出错: ${err.message}`);
    //     });
    // 执行git clone命令
    const cloneProcess = spawn('git', ['clone', '-b', branch, '--progress', repoUrl , projectDir]);

    // 监听标准输出以显示克隆进度
    cloneProcess.stdout.on('data', (data) => {
        process.stdout.write(data.toString());
    });

    // 监听标准错误以处理错误情况
    cloneProcess.stderr.on('data', (data) => {
        process.stderr.write(data.toString());
    });

    // 监听克隆命令的退出事件
    cloneProcess.on('close', (code) => {
        if (code === 0) {
            console.log(`项目已成功克隆到 ${projectDir}`);
        } else {
            console.error('克隆失败,请重试');
        }
    });
}

cloneRepo();